@@ -154,26 +154,69 @@ void SliceConnection::print_info(const std::string &tag) const
     std::cout << "covariance: " << covariance << std::endl;
 }
 
-Integrals::Integrals (const Polygons& polygons) {
+Integrals::Integrals(const Polygon &polygon)
+{
+    if (polygon.points.size() < 3) {
+        assert(false && "Polygon is expected to have non-zero area!");
+        *this = Integrals{};
+        return;
+    }
+    Vec2f p0 = unscaled(polygon.first_point()).cast<float>();
+    for (size_t i = 2; i < polygon.points.size(); i++) {
+        Vec2f p1 = unscaled(polygon.points[i - 1]).cast<float>();
+        Vec2f p2 = unscaled(polygon.points[i]).cast<float>();
+
+        float sign = cross2(p1 - p0, p2 - p1) > 0 ? 1.0f : -1.0f;
+
+        auto [area, first_moment_of_area, second_moment_area,
+              second_moment_of_area_covariance] = compute_moments_of_area_of_triangle(p0, p1, p2);
+
+        this->area += sign * area;
+        this->x_i += sign * first_moment_of_area;
+        this->x_i_squared += sign * second_moment_area;
+        this->xy += sign * second_moment_of_area_covariance;
+    }
+}
+
+Integrals::Integrals(const Polygons &polygons)
+{
     for (const Polygon &polygon : polygons) {
-        Vec2f p0 = unscaled(polygon.first_point()).cast<float>();
-        for (size_t i = 2; i < polygon.points.size(); i++) {
-            Vec2f p1 = unscaled(polygon.points[i - 1]).cast<float>();
-            Vec2f p2 = unscaled(polygon.points[i]).cast<float>();
+        *this = *this + Integrals{polygon};
+    }
+}
 
-            float sign = cross2(p1 - p0, p2 - p1) > 0 ? 1.0f : -1.0f;
+Integrals::Integrals(const Polylines& polylines, const std::vector<float>& widths) {
+    assert(polylines.size() == widths.size());
+    for (size_t i = 0; i < polylines.size(); ++i) {
+        Lines polyline{polylines[i].lines()};
+        float width{widths[i]};
+        for (const Line& line : polyline) {
+            Vec2f line_direction = unscaled(line.vector()).cast<float>();
+            Vec2f normal{line_direction.y(), -line_direction.x()};
+            normal.normalize();
 
-            auto [area, first_moment_of_area, second_moment_area,
-                  second_moment_of_area_covariance] = compute_moments_of_area_of_triangle(p0, p1, p2);
+            Vec2f line_a = unscaled(line.a).cast<float>();
+            Vec2f line_b = unscaled(line.b).cast<float>();
+            Vec2crd a = scaled(Vec2f{line_a + normal * width/2});
+            Vec2crd b = scaled(Vec2f{line_b + normal * width/2});
+            Vec2crd c = scaled(Vec2f{line_b - normal * width/2});
+            Vec2crd d = scaled(Vec2f{line_a - normal * width/2});
 
-            this->area += sign * area;
-            this->x_i += sign * first_moment_of_area;
-            this->x_i_squared += sign * second_moment_area;
-            this->xy += sign * second_moment_of_area_covariance;
+            const Polygon ractangle({a, b, c, d});
+            Integrals integrals{ractangle};
+            *this = *this + integrals;
         }
     }
 }
 
+Integrals::Integrals(float area, Vec2f x_i, Vec2f x_i_squared, float xy)
+    : area(area), x_i(std::move(x_i)), x_i_squared(std::move(x_i_squared)), xy(xy)
+{}
+
+Integrals operator+(const Integrals &a, const Integrals &b)
+{
+    return Integrals{a.area + b.area, a.x_i + b.x_i, a.x_i_squared + b.x_i_squared, a.xy + b.xy};
+}
 
 SliceConnection estimate_slice_connection(size_t slice_idx, const Layer *layer)
 {
@@ -473,18 +516,50 @@ ObjectPart::ObjectPart(
             continue;
         }
 
-        const Polygons polygons{collection->polygons_covered_by_width()};
+        for (const ExtrusionEntity* entity: collection->flatten()) {
+            Polylines polylines;
+            std::vector<float> widths;
 
-        const Integrals integrals{polygons};
-        const float volume = integrals.area * layer_height;
-        this->volume += volume;
-        this->volume_centroid_accumulator += to_3d(integrals.x_i, center_z * integrals.area) / integrals.area * volume;
+            if (
+                const auto* path = dynamic_cast<const ExtrusionPath*>(entity);
+                path != nullptr
+            ) {
+                polylines.push_back(path->as_polyline());
+                widths.push_back(path->width());
+            } else if (
+                const auto* loop = dynamic_cast<const ExtrusionLoop*>(entity);
+                loop != nullptr
+            ) {
+                for (const ExtrusionPath& path : loop->paths) {
+                    polylines.push_back(path.as_polyline());
+                    widths.push_back(path.width());
+                }
+            } else if (
+                const auto* multi_path = dynamic_cast<const ExtrusionMultiPath*>(entity);
+                multi_path != nullptr
+            ) {
+                for (const ExtrusionPath& path : multi_path->paths) {
+                    polylines.push_back(path.as_polyline());
+                    widths.push_back(path.width());
+                }
+            } else {
+                throw std::runtime_error(
+                    "Failed to construct object part from extrusions!"
+                    " Unknown extrusion type."
+                );
+            }
 
-        if (this->connected_to_bed) {
-            this->sticking_area += integrals.area;
-            this->sticking_centroid_accumulator += to_3d(integrals.x_i, bottom_z * integrals.area);
-            this->sticking_second_moment_of_area_accumulator += integrals.x_i_squared;
-            this->sticking_second_moment_of_area_covariance_accumulator += integrals.xy;
+            const Integrals integrals{polylines, widths};
+            const float volume = integrals.area * layer_height;
+            this->volume += volume;
+            this->volume_centroid_accumulator += to_3d(integrals.x_i, center_z * integrals.area) / integrals.area * volume;
+
+            if (this->connected_to_bed) {
+                this->sticking_area += integrals.area;
+                this->sticking_centroid_accumulator += to_3d(integrals.x_i, bottom_z * integrals.area);
+                this->sticking_second_moment_of_area_accumulator += integrals.x_i_squared;
+                this->sticking_second_moment_of_area_covariance_accumulator += integrals.xy;
+            }
         }
     }
 
